@@ -241,72 +241,59 @@ static const NAME_CODE protocol_table[] = {
 #define NAMEBUG(x)	#x, SSL_OP_##x
 static const LONG_NAME_MASK ssl_bug_tweaks[] = {
 
-#ifndef SSL_OP_MICROSOFT_SESS_ID_BUG
-#define SSL_OP_MICROSOFT_SESS_ID_BUG		0
+#if defined(SSL_OP_MICROSOFT_SESS_ID_BUG)
+    NAMEBUG(MICROSOFT_SESS_ID_BUG),	/* 0x00000001L */
 #endif
-    NAMEBUG(MICROSOFT_SESS_ID_BUG),
 
-#ifndef SSL_OP_NETSCAPE_CHALLENGE_BUG
-#define SSL_OP_NETSCAPE_CHALLENGE_BUG		0
+#if defined(SSL_OP_NETSCAPE_CHALLENGE_BUG)
+    NAMEBUG(NETSCAPE_CHALLENGE_BUG),	/* 0x00000002L */
 #endif
-    NAMEBUG(NETSCAPE_CHALLENGE_BUG),
 
-#ifndef SSL_OP_LEGACY_SERVER_CONNECT
-#define SSL_OP_LEGACY_SERVER_CONNECT		0
+#if defined(SSL_OP_LEGACY_SERVER_CONNECT)
+    NAMEBUG(LEGACY_SERVER_CONNECT),	/* 0x00000004L */
 #endif
-    NAMEBUG(LEGACY_SERVER_CONNECT),
 
-#ifndef SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG
-#define SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG 0
-#endif
-    NAMEBUG(NETSCAPE_REUSE_CIPHER_CHANGE_BUG),
+#if defined(SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG)
+    NAMEBUG(NETSCAPE_REUSE_CIPHER_CHANGE_BUG),	/* 0x00000008L */
     "CVE-2010-4180", SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG,
-
-#ifndef SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG
-#define SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG	0
 #endif
-    NAMEBUG(SSLREF2_REUSE_CERT_TYPE_BUG),
 
-#ifndef SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER
-#define SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER	0
+#if defined(SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG)
+    NAMEBUG(SSLREF2_REUSE_CERT_TYPE_BUG),	/* 0x00000010L */
 #endif
-    NAMEBUG(MICROSOFT_BIG_SSLV3_BUFFER),
 
-#ifndef SSL_OP_MSIE_SSLV2_RSA_PADDING
-#define SSL_OP_MSIE_SSLV2_RSA_PADDING		0
+#if defined(SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER)
+    NAMEBUG(MICROSOFT_BIG_SSLV3_BUFFER),/* 0x00000020L	 */
 #endif
-    NAMEBUG(MSIE_SSLV2_RSA_PADDING),
+
+#if defined(SSL_OP_MSIE_SSLV2_RSA_PADDING)
+    NAMEBUG(MSIE_SSLV2_RSA_PADDING),	/* 0x00000040L */
     "CVE-2005-2969", SSL_OP_MSIE_SSLV2_RSA_PADDING,
-
-#ifndef SSL_OP_SSLEAY_080_CLIENT_DH_BUG
-#define SSL_OP_SSLEAY_080_CLIENT_DH_BUG		0
 #endif
-    NAMEBUG(SSLEAY_080_CLIENT_DH_BUG),
 
-#ifndef SSL_OP_TLS_D5_BUG
-#define SSL_OP_TLS_D5_BUG			0
+#if defined(SSL_OP_SSLEAY_080_CLIENT_DH_BUG)
+    NAMEBUG(SSLEAY_080_CLIENT_DH_BUG),	/* 0x00000080L */
 #endif
-    NAMEBUG(TLS_D5_BUG),
 
-#ifndef SSL_OP_TLS_BLOCK_PADDING_BUG
-#define SSL_OP_TLS_BLOCK_PADDING_BUG		0
+#if defined(SSL_OP_TLS_D5_BUG)
+    NAMEBUG(TLS_D5_BUG),		/* 0x00000100L	 */
 #endif
-    NAMEBUG(TLS_BLOCK_PADDING_BUG),
 
-#ifndef SSL_OP_TLS_ROLLBACK_BUG
-#define SSL_OP_TLS_ROLLBACK_BUG			0
+#if defined(SSL_OP_TLS_BLOCK_PADDING_BUG)
+    NAMEBUG(TLS_BLOCK_PADDING_BUG),	/* 0x00000200L */
 #endif
-    NAMEBUG(TLS_ROLLBACK_BUG),
 
-#ifndef SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS
-#define SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS	0
+#if defined(SSL_OP_TLS_ROLLBACK_BUG)
+    NAMEBUG(TLS_ROLLBACK_BUG),		/* 0x00000400L */
 #endif
-    NAMEBUG(DONT_INSERT_EMPTY_FRAGMENTS),
 
-#ifndef SSL_OP_CRYPTOPRO_TLSEXT_BUG
-#define SSL_OP_CRYPTOPRO_TLSEXT_BUG		0
+#if defined(SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS)
+    NAMEBUG(DONT_INSERT_EMPTY_FRAGMENTS),	/* 0x00000800L */
+#endif
+
+#if defined(SSL_OP_CRYPTOPRO_TLSEXT_BUG)
+    NAMEBUG(CRYPTOPRO_TLSEXT_BUG),	/* 0x80000000L */
 #endif
-    NAMEBUG(CRYPTOPRO_TLSEXT_BUG),
     0, 0,
 };
 
@@ -517,6 +504,11 @@ int     tls_protocol_mask(const char *plist)
     int     exclude = 0;
     int     include = 0;
 
+#define FREE_AND_RETURN(ptr, res) do { \
+	myfree(ptr); \
+	return (res); \
+    } while (0)
+
     save = cp = mystrdup(plist);
     while ((tok = mystrtok(&cp, "\t\n\r ,:")) != 0) {
 	if (*tok == '!')
@@ -525,12 +517,9 @@ int     tls_protocol_mask(const char *plist)
 	else
 	    include |= code =
 		name_code(protocol_table, NAME_CODE_FLAG_NONE, tok);
-	if (code == TLS_PROTOCOL_INVALID) {
-	    myfree(save);
-	    return TLS_PROTOCOL_INVALID;
-	}
+	if (code == TLS_PROTOCOL_INVALID)
+	    FREE_AND_RETURN(save, TLS_PROTOCOL_INVALID);
     }
-    myfree(save);
 
     /*
      * When the include list is empty, use only the explicit exclusions.
@@ -539,7 +528,8 @@ int     tls_protocol_mask(const char *plist)
      * we don't know about at compile time, and this is unavoidable because
      * the OpenSSL API works with compile-time *exclusion* bit-masks.
      */
-    return (include ? (exclude | (TLS_KNOWN_PROTOCOLS & ~include)) : exclude);
+    FREE_AND_RETURN(save,
+	(include ? (exclude | (TLS_KNOWN_PROTOCOLS & ~include)) : exclude));
 }
 
 /* tls_param_init - Load TLS related config parameters */
@@ -879,8 +869,7 @@ long    tls_bug_bits(void)
 {
     long    bits = SSL_OP_ALL;		/* Work around all known bugs */
 
-#if OPENSSL_VERSION_NUMBER >= 0x00908000L && \
-	OPENSSL_VERSION_NUMBER < 0x10000000L
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
     long    lib_version = SSLeay();
 
     /*
@@ -906,10 +895,6 @@ long    tls_bug_bits(void)
 	bits &= ~long_name_mask_opt(VAR_TLS_BUG_TWEAKS, ssl_bug_tweaks,
 				    var_tls_bug_tweaks, NAME_MASK_ANY_CASE |
 				    NAME_MASK_NUMBER | NAME_MASK_WARN);
-#ifdef SSL_OP_SAFARI_ECDHE_ECDSA_BUG
-	/* Not relevant to SMTP */
-	bits &= ~SSL_OP_SAFARI_ECDHE_ECDSA_BUG;
-#endif
     }
     return (bits);
 }
